@@ -250,7 +250,10 @@ JSON_GOI18N_V2_SIMPLE = """{
         "description": "a piece or strip of strong paper, plastic, metal, leather, etc., for attaching by one end to something as a mark or label",
         "one": "{{.count}} tag",
         "other": "{{.count}} tags"
-    }
+    },
+    "nested.key.hello": "world",
+    "nested.key.world": "hello",
+    ".world": "hello"
 }
 """
 
@@ -266,6 +269,15 @@ JSON_GOI18N_V2_COMPLEXE = """{
         "description": "a piece or strip of strong paper, plastic, metal, leather, etc., for attaching by one end to something as a mark or label",
         "one": "{{.count}} tag",
         "other": "{{.count}} tags"
+    },
+    "nested.key.hello": {
+        "other": "world"
+    },
+    "nested.key.world": {
+        "other": "hello"
+    },
+    ".world": {
+        "other": "hello"
     }
 }
 """
@@ -734,6 +746,17 @@ class TestJSONNestedResourceStore(test_monolingual.TestMonolingualUnit):
         from_string = id_class.from_string
         assert from_string(id_string) == id_class(expected)
 
+    def test_dot_keys(self):
+        jsontext = """{
+    "key.dot": "message"
+}
+"""
+        store = self.StoreClass()
+        store.parse(jsontext)
+        # Edit target
+        store.units[0].target = "message"
+        assert bytes(store).decode() == jsontext
+
 
 class TestWebExtensionUnit(test_monolingual.TestMonolingualUnit):
     UnitClass = jsonl10n.WebExtensionJsonUnit
@@ -753,6 +776,29 @@ class TestWebExtensionStore(test_monolingual.TestMonolingualStore):
             == b'{\n    "key": {\n        "message": "value",\n        "description": "note"\n    }\n}\n'
         )
 
+    def test_dot_keys(self):
+        store = self.StoreClass()
+        store.parse('{"key.dot": {"message": "value", "description": "note"}}')
+        out = BytesIO()
+        store.serialize(out)
+
+        assert (
+            out.getvalue()
+            == b'{\n    "key.dot": {\n        "message": "value",\n        "description": "note"\n    }\n}\n'
+        )
+
+    def test_leading_dot_keys(self):
+        jsontext = """{
+    ".dot": {
+        "message": "value",
+        "description": "note"
+    }
+}
+"""
+        store = self.StoreClass()
+        store.parse(jsontext)
+        assert bytes(store).decode() == jsontext
+
     def test_serialize_no_description(self):
         store = self.StoreClass()
         store.parse('{"key": {"message": "value"}}')
@@ -1233,6 +1279,20 @@ class TestGoI18NJsonFile(test_monolingual.TestMonolingualStore):
         with raises(ValueError):
             store.parse(JSON_I18NEXT_PLURAL)
 
+    def test_dot_keys(self):
+        jsontext = """[
+    {
+        "id": ".table",
+        "translation": "message"
+    }
+]
+"""
+        store = self.StoreClass()
+        store.parse(jsontext)
+        # Edit target
+        store.units[0].target = "message"
+        assert bytes(store).decode() == jsontext
+
 
 class TestGoI18NV2JsonFile(test_monolingual.TestMonolingualStore):
     StoreClass = jsonl10n.GoI18NV2JsonFile
@@ -1241,12 +1301,14 @@ class TestGoI18NV2JsonFile(test_monolingual.TestMonolingualStore):
         store = self.StoreClass()
         store.parse(JSON_GOI18N_V2_SIMPLE)
 
-        assert len(store.units) == 3
+        assert len(store.units) == 6
         assert store.units[0].target == "value"
         assert store.units[1].target == "Table"
         assert store.units[2].target == multistring(
             ["{{.count}} tag", "{{.count}} tags"]
         )
+        assert store.units[3].target == "world"
+        assert store.units[4].target == "hello"
 
         assert bytes(store).decode() == JSON_GOI18N_V2_SIMPLE
 
@@ -1295,6 +1357,11 @@ class TestGoI18NV2JsonFile(test_monolingual.TestMonolingualStore):
         store = self.StoreClass()
         store.parse(JSON_GOI18N_V2_SIMPLE)
 
+        assert len(store.units) == 6
+
+        store.units[2].target = multistring(["{{.count}} tag", "{{.count}} tags"])
+        assert bytes(store).decode() == JSON_GOI18N_V2_SIMPLE
+
         store.units[2].target = multistring(["{{.count}} tag"])
 
         assert '"other": "{{.count}} tag"' in bytes(store).decode()
@@ -1303,12 +1370,14 @@ class TestGoI18NV2JsonFile(test_monolingual.TestMonolingualStore):
         store = self.StoreClass()
         store.parse(JSON_GOI18N_V2_COMPLEXE)
 
-        assert len(store.units) == 3
+        assert len(store.units) == 6
         assert store.units[0].target == "value"
         assert store.units[1].target == "Table"
         assert store.units[2].target == multistring(
             ["{{.count}} tag", "{{.count}} tags"]
         )
+        assert store.units[3].target == "world"
+        assert store.units[4].target == "hello"
 
         assert bytes(store).decode() == JSON_GOI18N_V2_SIMPLE
 
@@ -1317,6 +1386,17 @@ class TestGoI18NV2JsonFile(test_monolingual.TestMonolingualStore):
         with raises(ValueError):
             store.parse(JSON_I18NEXT_PLURAL)
 
+    def test_dot_keys(self):
+        jsontext = """{
+    ".table": "message"
+}
+"""
+        store = self.StoreClass()
+        store.parse(jsontext)
+        # Edit target
+        store.units[0].target = "message"
+        assert bytes(store).decode() == jsontext
+
 
 class TestARBJsonFile(test_monolingual.TestMonolingualStore):
     StoreClass = jsonl10n.ARBJsonFile
@@ -1333,6 +1413,16 @@ class TestARBJsonFile(test_monolingual.TestMonolingualStore):
 
         assert bytes(store).decode() == JSON_ARB.decode()
 
+    def test_leading_dot_keys(self):
+        jsontext = """{
+  ".dot": "dot",
+  "@.dot": {}
+}
+"""
+        store = self.StoreClass()
+        store.parse(jsontext)
+        assert bytes(store).decode() == jsontext
+
 
 JSON_FORMATJS = """{
     "hak27d": {
@@ -1368,3 +1458,15 @@ class TestFormatJSJsonFile(test_monolingual.TestMonolingualStore):
         assert store.units[3].getnotes() == "placeholder text"
 
         assert bytes(store).decode() == JSON_FORMATJS
+
+    def test_leading_dot_keys(self):
+        jsontext = """{
+    ".dot": {
+        "defaultMessage": "Control Panel",
+        "description": "title of control panel section"
+    }
+}
+"""
+        store = self.StoreClass()
+        store.parse(jsontext)
+        assert bytes(store).decode() == jsontext
